
pro eit_proton_summary, time0, time1, $
   wave=wave, full=full, $                      ; pass to > eit_files
   eitfiles=eitfiles,    $                      ; optional filelist
   cadence=cadence, $                           ; optional cadence (mins_
   subfov=subfov, threshold=threshold, $        ; pass to > cosmic_stat
   nodisplay=nodisplay, www=www, movie=movie, $ ; display & www summary options
   quiet=quiet, cstats=cstats, index=index, $
   mwindow=mwindow, except=except, $
   ll=ll,lr=lr,ul=ul,ur=ur, center=center, pixsub=pixsub, _extra=_extra, $
   l1q=l1q
;+

;   Name: eit_proton_summary
;
;   Purpose: "standard" processing pipeline/summary plot for proton events
;
;   Calls:
;      plot_goes, plot_goesp, eit_files, eit_prep, cosmic_stat 
;      time_window, evt_grid, plus the usual ssw suspects.
;
;   Input Parameters:
;      time0, time1 - time range of interest
;      wave - desired wavelength (default = 195)
;      full - if set, use the FULL DISK data base
;      subfov - desired box for cosmic ray counts - per cosmic_stat.pro
;               [default = [0,0,1024,50] 
;      threshold - per cosmic_stat.pro
;      nodisplay - if set, dont make the summary plots (just return cstat)
;      mwindow - time window (minutes) +/-  around EIT Peak for movie
;      cstats (OUTPUT) - the affected area statistics (%pixels affected)
;      index  (OUTPUT) - index (header) vector
;-
;

if not keyword_set(wave) then wave='195'
full=keyword_set(full)
nodisplay=keyword_set(nodisplay)
display=1-nodisplay
movie=keyword_set(movie)
www=(keyword_set(www) or movie) and display
quiet=keyword_set(quiet)
l1q=keyword_set(l1q)
; -------- find the files of interest 
case 1 of 
   data_chk(eitfiles,/string): efiles=eitfiles           ; user supplied list
   l1q: begin 
         efiles=eit_high_cadence(time0,time1,/l1q,/return)
         sscnt=n_elements(efiles)
         ss=lindgen(sscnt)
   endcase 

   n_params() eq 2: begin
          
      if full then begin
         efiles=eit_files(time0,time1,full=full, wave=wave)
         sscnt=([n_elements(efiles),0])(efiles(0) eq '')
      endif else begin 
      efiles=eit_files(time0,time1)
      read_eit, efiles, header
      ss=where(gt_tagval(header,/naxis1) eq 512 and $
               gt_tagval(header,/naxis2) eq 512 and $
               gt_tagval(header,/wavelnth) eq strtrim(wave,2) and $
               gt_tagval(header,/nmissb) le 1    and $
               gt_tagval(header,/object,missing='') eq "full FOV", sscnt)
       if sscnt lt 10 then $
                ss=where(gt_tagval(header,/naxis1) eq 1024 and $
               gt_tagval(header,/naxis2) eq 1024 and $
               gt_tagval(header,/wavelnth) eq strtrim(wave,2) and $
               gt_tagval(header,/nmissb) le 1    and $
               gt_tagval(header,/object,missing='') eq "full FOV", sscnt) 
       efiles=efiles(ss)
      endelse
   endcase
   else: begin 
      box_message,['Must supply time range or EITFILES list',$
                   'IDL> eit_proton_summary,time0,time1 [OPTIONS]', $
                   '     -OR-',$
                   'IDL> eit_proton_summary,eitfiles=FILELIST [OPTIONS]']
      return
   endcase
endcase

if sscnt eq 0 then begin 
   box_message,"No eit files match desired criteria, returning..."
   return
endif

if keyword_set(cadence) then begin 
   ss=grid_data(file2time(efiles),minutes=cadence)
endif else if n_elements(ss) eq 0 then ss=lindgen(n_elements(efiles))

if keyword_set(except) then begin
   ftimes=file2time(efiles(ss),out='int')
   sss=tim2dset(ftimes,except,delta=dt)
   remove,sss,ss
endif

read_eit, efiles(ss), index                      ; read headers
ss=sort(anytim(index))
;ss=where(index.naxis1 eq 512)

outnames=concat_dir('$path_http','eitp'+time2file(index)+'.fits')

if l1q then begin 
   read_eit,efiles(ss),pindex,pdata
endif else begin 
   for i=0,n_elements(ss)-1 do eit_prep, efiles(ss(i)),outfits=outnames(i)
   mreadfits,outnames,pindex,pdata,outsize=512
endelse

pindex=struct2ssw(pindex)

mreadfits_fixup,pindex,pdata
delvarx,data                                     ; memory cleanup

if not keyword_set(subfov) then subfov=[0,0,data_chk(pdata,/nx),50]
if not keyword_set(threshold) then threshold=20

if not quiet then box_message,$
   'Using subfield boxsize [x0,y0,nx,ny] >> [' + arr2str(subfov,/trim) + ']'


cstats=cosmic_stat(pindex, pdata, threshold, /percent, $
                   peak_spectra=peak_spectra, subfov=subfov)

psave=!p.multi
time_window,index, t0, t1
deit=deriv_arr([cstats(0),cstats])
epeakd=(where(deit eq max(deit)))(0)

epeak=(where(cstats eq max(cstats)))(0)
epeak50=where(cstats ge .5*cstats(epeak))  ; .5 max
epeakd=last_nelem(epeak50)

rd_goesx_ascii,t0,t1,/goes10,/one_min,gxd
gpeak=(where(gxd.lo eq max(gxd.lo)))(0)
dgoes=deriv_arr([gxd(0).lo,gxd.lo])
gpeakd=(where(dgoes eq max(dgoes)))(0)

dgp2ep=ssw_deltat(pindex(epeak),ref=gxd(gpeak),/hour)
dgr2er=ssw_deltat(pindex(epeakd), ref=gxd(gpeakd),/hour)

if display then begin 
   wdef,xx,zbuffer=www, 1280,1024
   !p.multi=[0,1,6]
   linecolors
   plot_goes,  t0,  t1, background=11, color=7, xstyle=1, charsize=1.5, $
      ymargin=[2,3],xmargin=[8,0], timerange=[t0,t1],/nolabel,/ascii, $
      gcolor=100 
   evt_grid,pindex(epeakd),color=5,linestyle=0, thick=2, label='EIT Peak Event Half Max'+$
      anytim(pindex(epeakd),/vms,/trunc),labpos=.95,align=0

   plot_goesp, t0, t1,  xstyle=1, /nowindow, charsize=1.5, /log, $
      ymargin=[2,3], xmargin=[8,0], /proton_only,timerange=[t0,t1], $
      yrange=[.1,10e4],/ystyle,/nolabel, symsize=.5

   plot_ace,t0,t1,xsytle=1,/ytype,ymargin=[2,3],xmargin=[8,0],/epam, $
     charsize=1.5,/nowindow,xstyle=1,yrange=[1,10e4],/ystyle,/nolabel,/proton

   plot_ace,t0,t1,xsytle=1,/ytype,ymargin=[2,3],xmargin=[8,0],/sis, $
     charsize=1.5,/nowindow,xstyle=1,yrange=[1,10e4],/ystyle,/nolabel

   plot_ace,t0,t1,xsytle=1,/ytype,ymargin=[2,3],xmargin=[8,0],/epam, $
     charsize=1.5,/nowindow,xstyle=1,yrange=[1,10e4],/ystyle,/nolabel,/electron
   utplot, index, cstats, ytitle='Affected Area (%Pixels)', $
                      timerange=[t0,t1],xstyle=1, psym=2, color=5, charsize=1.5, $
      ymargin=[4,3], xmargin=[8,0], title='EIT CCD Affected Area'
endif

!p.multi=psave

if www then begin 
   root='eit_protons_' + time2file(t0)
   gdir=concat_dir('$path_http','protons')
   if not file_exist(gdir) then gdir=get_logenv('path_http')
   gname=concat_dir(gdir,root+'.gif')                           ; summary plot
   hdoc=concat_dir(gdir,root+'.html')                           ; top http
   zbuff2file,gname                                             ; Z->gif
loadct,0
   movie_html=''  
   cfile=root+'_cadence'
   sfile=root+'_peak_spectra.gif'
   sname=concat_dir(gdir,sfile)
   pfile=root+'_peak_events.gif'
   pname=concat_dir(gdir,pfile)
   genxfile=concat_dir(gdir,root+'.genx')

;  --- make peak event image ------
   wdef,xx,im=pdata(*,*,epeak),/zbuffer
   ct2rgb,3,gamma=.8
   tvscl,safe_log10(pdata(*,*,epeak)<3000,/byte)
   zbuff2file,pname
   ptf=str_replace(pname,'.gif','_thumb.gif')
   pthumb=mkthumb(ingif=pname,outfile=ptf,nx=200)

;  --- plot Peak spectra ---------
   wdef,xx,512,512,/zbuffer
   linecolors
   plot, peak_spectra, psym=10, background=11,color=7,$
      title='Energy Spectra for Image @' + anytim(pindex(epeak),/vms,/trunc), $
      ytitle='Frequency (#Pixels)', xtitle='CCD DN'

   zbuff2file,sname
   stf=str_replace(sname,'.gif','_thumb.gif')
   sthumb=mkthumb(ingif=sname,outfile=stf,nx=200)
   
   thumbhtml=thumbnail_table_html([pname,sname],[pthumb,sthumb])

;make a cadence plot to append to movie -----------
   cname=concat_dir(gdir,cfile+'.gif')
   wdef,xx,/zbuffer,data_chk(pdata,/nx),data_chk(pdata,/nx)/3
   wdef,xx,/zbuffer,512,512/3
   loadct,0
   !p.multi=0
    utplot, index, cstats, ytitle='EIT CCD Affected Area (%Pixels)', $
          timerange=[t0,t1],xstyle=1, psym=2, color=0, charsize=1, $
          ymargin=[6,2], xmargin=[8,0], background=!p.color-10, $
          mtitle='EIT Affected Area'

   evt_grid,index,ticklen=.02,tickpos=.95,color=5,thick=2,linestyle=0
   
   evt_grid,index(epeakd),label='EIT Peak Rise (half max)'+anytim(index(epeakd),/vms,/trunc),$
      align=0,color=50,thick=2, tickpos=.99, labpos=.88,labcolor=20,/arrow
   evt_grid,index(epeakd),color=20
   zbuff2file,cname
   eit_fill_cube, pdata 
   if movie then begin 
       if n_elements(mwindow) ne 2 then mwindow=[-1,3]
       time_window,[anytim(gxd(gpeakd),/vms), $
                    anytim(pindex(epeak),/vms)],mt0, mt1, hour=mwindow
       mss=sel_timrange(pindex,mt0,mt1)
       mss=indgen(n_elements(pindex))
;       pdata=cube_edit(temporary(pdata),/data,ss=mss)
;       pindex=pindex(mss)
       box_message,'Cleaning data via <ssw_unspike_cube>'
       rfac=([.5,.25])(data_chk(pdata,/nx) eq 1024)
           stop,'fillcube pdata' 
       ssw_fill_cube,pdata>2, missing=2
;stop,'subimage'
       ssw_subimage,pindex,pdata,cindex,cdata, $
          ll=ll,lr=lr,ur=ur,ul=ul,center=center, pixsub=pixsub

       cleandata=ssw_unspike_cube(cindex,cdata)
;       cleandata=confac(temporary(ssw_unspike_cube(pindex,pdata)),rfac)
;       difference_movie,pindex,cleandata,cindex,dcleandata
;       delvarx,cleandata
;       dcleandata=[[[dcleandata]],[[last_nelem(dcleandata)]]] 
       rfac=float(data_chk(cleandata,/nx)) / float(data_chk(pdata,/nx))
       pdata=confac(temporary(pdata),rfac)
       ;mdata=[(safe_log10(pindex,pdata,/byte)), $
       ;        (safe_log10(cindex,cleandata,/byte))]
       mdata=[bytscl(do_eit_scaling(pdata,/no_prep,/log)), $
              !p.color-bytscl(do_eit_scaling(cleandata,/no_prep,/log))]
 
                   ; bytscl(dcleandata,min=-200,max=200)]
       mreadfits_fixup,pindex,mdata

       wdef,xx,data_chk(mdata,/nx),300,/zbuffer
       time_window,pindex,gt0,gt1,minute=30 
       pmtemp=!p
       !p.multi=[0,1,2]
       plot_ace,gt0,gt1,/epam,/proton,/nowindow, $
          ymargin=[2,3],xmargin=[8,0],/xstyle,charsize=.7
       plot_goes,gt0,gt1,color=7 , /ascii, gcolor=20, $
          ymargin=[2,3],xmargin=[8,0],/xstyle, charsize=.7
       linecolors
       split_colortab,3,!p.color-15,16,/reverse
       tvlct,r,g,b,/get
       event_movie3, pindex, moviedata ; data=(mdata+16)<!p.color,moviedata
       ;mdata=bytscl(!p.color-fix(mdata),top=!p.color-16)
       mdata=bytscl(fix(mdata),top=!p.color-16)
       moviedata=[[mdata+16],[moviedata]]
       !p=pmtemp

       special_movie,pindex,moviedata,/inc_times,html=movie_html,$    
          movie_dir=gdir,thumbsize=160, /no_htmldoc, movie_name=root,$
          r,g,b,_extra=_extra
   endif
   html_doc,hdoc,/header
   file_append,hdoc,'<h3> EIT Proton Event Summary</h3>'
   file_append,hdoc,['Created by <em>eit_proton_summary.pro</em> ', $
                  ' at ' + systime(),'<p>']

   header=['Start Time', 'Stop Time',$
           'GOES XRay Peak Time',$          
           'Maximum EIT Affected Area',$
           'dT(GOES XR Peak to EIT Peak)', $
           'dt(GOES XR Rise to EIT Rise)', $ 
           'Median Threshold','CCD Sub Field [x0,y0,nx,ny]']
   tdata=[anytim([time0,time1],/ecs,/trunc),$
          anytim(gxd(gpeak),/ecs,/trunc), $
          string(max(cstats),format='(f5.1)') +'%', $
          string(dgp2ep,format='(f5.1)') +' (hr)',$
          string(dgr2er,format='(f5.1)') +' (hr)',$          
          string(threshold,format='(F5.1)'), $
          '[ ' + arr2str(subfov,/trim) + ' ]']
   file_append,hdoc,strtab2html([[header],[tdata]],/row0) 
   file_append,hdoc,['<p>',$
     '<h3>Event Movie - Original full disk and despiked sub field</h3>',$
      movie_html]
   
   gtab=transpose(['GOES X-Ray / PROTONS / EIT Affected Area', $
                   '<img src="' + root + '.gif">' ])
   file_append,hdoc,strtab2html(gtab,/row0)
   file_append,hdoc,['<p>',$
     '<h3> Image and Spectra at time of EIT event peak: '+ $
      anytim(index(epeak),/ecs) +   '</h3>',$
     thumbhtml]
   html_doc,hdoc,/trailer

   savegen,file=genxfile,root,time0,time1,threshold,subfov,index,pindex,cstats
  
endif

if file_exist(html_doc) then begin 
   mail,http_names(html_doc),/no_def,subj='EIT  PROTONS Completed',$
      user=get_user()
endif

return
end
